@@ -6,7 +6,8 @@ from dbmi_client import authn
 from dbmi_client import authz
 
 # Get the app logger
-logger = dbmi_settings.get_logger()
+import logging
+logger = logging.getLogger(dbmi_settings.LOGGER_NAME)
 
 
 def dbmi_user(view):
@@ -21,7 +22,7 @@ def dbmi_user(view):
 
         # Check for current user
         if not request.user or not request.user.is_authenticated:
-            return authn.logout_redirect(request)
+            return authn.login_redirect(request)
 
         # Let it go
         return view(request, *args, **kwargs)
@@ -41,7 +42,7 @@ def dbmi_admin(view):
 
         # Check for current user
         if not request.user or not request.user.is_authenticated:
-            return authn.logout_redirect(request)
+            return authn.login_redirect(request)
 
         # Get the payload
         payload = authn.get_jwt_payload(request, verify=False)
@@ -83,7 +84,7 @@ def dbmi_group(group):
 
             # Check for current user
             if not request.user or not request.user.is_authenticated:
-                return authn.logout_redirect(request)
+                return authn.login_redirect(request)
 
             # Get the payload
             payload = authn.get_jwt_payload(request, verify=False)
@@ -121,7 +122,7 @@ def dbmi_role(role):
 
             # Check for current user
             if not request.user or not request.user.is_authenticated:
-                return authn.logout_redirect(request)
+                return authn.login_redirect(request)
 
             # Get the payload
             payload = authn.get_jwt_payload(request, verify=False)
@@ -159,7 +160,7 @@ def dbmi_app_permission(permission):
 
             # Check for current user
             if not request.user or not request.user.is_authenticated:
-                return authn.logout_redirect(request)
+                return authn.login_redirect(request)
 
             # Get the payload
             payload = authn.get_jwt_payload(request, verify=False)
@@ -206,7 +207,7 @@ def dbmi_item_permission(item, permission):
 
             # Check for current user
             if not request.user or not request.user.is_authenticated:
-                return authn.logout_redirect(request)
+                return authn.login_redirect(request)
 
             # Get their email address
             email = authn.get_jwt_email(request, verify=False)
