@@ -45,10 +45,12 @@ CONFIG_DEFAULTS = {
     'AUTHN_URL': None,
     'AUTHZ_URL': None,
     'REG_URL': None,
-    'JWT_AUTHZ_NAMESPACE': None,
+    'FILESERVICE_URL': None,
 
     # Optionally disable logging
+    'LOGGER_NAME': 'dbmi_client',
     'ENABLE_LOGGING': True,
+    'LOG_LEVEL': logging.WARNING,
 
     # Universal login screen branding
     'AUTHN_TITLE': 'DBMI Client',
@@ -60,6 +62,7 @@ CONFIG_DEFAULTS = {
     'AUTHZ_USER_GROUP': 'dbmi-user',
 
     # JWT bits
+    'JWT_AUTHZ_NAMESPACE': None,
     'JWT_HTTP_PREFIX': 'JWT ',
     'JWT_COOKIE_NAME': 'DBMI_JWT',
     'JWT_COOKIE_DOMAIN': '.dbmi.hms.harvard.edu',
@@ -70,6 +73,12 @@ CONFIG_DEFAULTS = {
     'AUTH0_TENANT': None,
     'AUTH0_SCOPE': 'openid email',
 
+    # Auth0 clients
+    'AUTH0_CLIENTS': None,  # Use this dictionary to specify multiple auth0 tenant: {client ID, secret} configs
+
+    # Auth0 tenants
+    'AUTH0_TENANTS': None,  # Use this list to specify Auth0 tenants whose clients should be considered valid here
+
     # Configurations surrounding usage of a local Django user model
     'USER_MODEL_ENABLED': False,
     'USER_MODEL_AUTOCREATE': True,
@@ -78,11 +87,18 @@ CONFIG_DEFAULTS = {
     'DRF_OBJECT_OWNER_KEY': 'user',
 
     # Filservice
+    'FILESERVICE_TOKEN': None,  # The authentication token to be used with Fileservice
     'FILESERVICE_BUCKETS': [],  # The name of the S3 buckets Fileservice should use
     'FILESERVICE_GROUP': None,  # Typically would be the same as CLIENT
+    'FILESERVICE_ADMINS': [],  # A list of Fileservice users and/or user emails that should administer group
 
     # Login settings
     'LOGIN_REDIRECT_KEY': 'next',  # The query parameter key specifying where logged in users should be sent
+    'LOGIN_REDIRECT_URL': None,  # The post-login URL to send users to if not specified by 'LOGIN_REDIRECT_KEY'
+
+    # Logout settings
+    'LOGOUT_REDIRECT_KEY': 'next',  # The query parameter key specifying where logged out users should be sent
+    'LOGOUT_REDIRECT_URL': None,  # The post-logout URL to send users to if not specified by 'LOGOUT_REDIRECT_KEY'
 }
 
 # List of settings that cannot be defaulted and must be user-defined
@@ -117,7 +133,8 @@ class DBMISettings(object):
 
                 # Update the client config with pre-defined environment URLs, etc
                 if user_settings.get('ENVIRONMENT') in DBMI_ENVIRONMENTS:
-                    user_settings.update(DBMI_ENVIRONMENTS[user_settings.get('ENVIRONMENT')])
+                    env_settings = DBMI_ENVIRONMENTS[user_settings.get('ENVIRONMENT')]
+                    user_settings.update({k: v for k, v in env_settings.items() if k not in user_settings})
 
                 else:
                     # Check for them in environment
@@ -164,8 +181,20 @@ class DBMISettings(object):
         if 'CLIENT' not in user_settings:
             raise AttributeError('CLIENT configuration must be set')
 
-        if 'AUTH0_TENANT' not in user_settings or 'AUTH0_CLIENT_ID' not in user_settings:
-            raise AttributeError('AUTH0_TENANT and AUTH0_CLIENT_ID configurations must be set')
+        # Check possible Auth0 configurations (single client, multiple clients, multiple tenants)
+        if ('AUTH0_TENANT' not in user_settings or 'AUTH0_CLIENT_ID' not in user_settings) \
+                and 'AUTH0_TENANTS' not in user_settings \
+                and 'AUTH0_CLIENTS' not in user_settings:
+            raise AttributeError('One of AUTH0_TENANT/AUTH0_CLIENT_ID, AUTH0_CLIENTS or '
+                                 'AUTH0_TENANTS configurations must be set')
+
+        # If Auth0 login enabled, ensure we've got needed Auth0 configurations
+        if 'dbmi_client.login' in settings.INSTALLED_APPS \
+            and ('AUTH0_TENANT' not in user_settings
+                 or 'AUTH0_CLIENT_ID' not in user_settings
+                 or 'AUTH0_SECRET' not in user_settings):
+            raise AttributeError('AUTH0_TENANT/AUTH0_CLIENT_ID/AUTH0_SECRET must be specified for '
+                                 'the "dbmi_client.login" app to function')
 
         # Ensure environment is set and if not prod or dev, ensure service URLs are provided
         if 'ENVIRONMENT' in user_settings and \
@@ -197,6 +226,24 @@ class DBMISettings(object):
             if missing_urls:
                 raise AttributeError('{} configuration(s) must be set'.format(missing_urls))
 
+            if 'FILESERVICE_URL' not in user_settings:
+                # Check environment
+                if os.environ.get('DBMI_FILESERVICE_URL'):
+                    user_settings['FILESERVICE_URL'] = os.environ.get('DBMI_FILESERVICE_URL')
+                else:
+                    warnings.warn("FILESERVICE_URL is not set, Fileservice requests will fail", ResourceWarning)
+
+            if ('FILESERVICE_GROUP' in user_settings or
+                'FILESERVICE_BUCKETS' in user_settings or
+                'FILESERVICE_ADMINS' in user_settings or
+                'FILESERVICE_TOKEN' in user_settings) and \
+                    ('FILESERVICE_GROUP' not in user_settings or
+                     'FILESERVICE_BUCKETS' not in user_settings or
+                     'FILESERVICE_ADMINS' not in user_settings or
+                     'FILESERVICE_TOKEN' not in user_settings):
+                    warnings.warn("FILESERVICE_GROUP, FILESERVICE_BUCKETS, FILESERVICE_ADMINS and FILESERVICE_TOKEN must be "
+                                  "defined for Fileservice usage", ResourceWarning)
+
             if 'JWT_AUTHZ_NAMESPACE' not in user_settings:
                 warnings.warn(
                     "JWT_AUTHZ_NAMESPACE is not set, JWT claims will not be inspected "
@@ -239,6 +286,7 @@ class DBMISettings(object):
 
             # Else, return true if the client has added either backend classes to AUTHENTICATION_BACKENDS
             return 'dbmi_client.authn.DBMIModelAuthenticationBackend' in settings.AUTHENTICATION_BACKENDS or \
+                   'dbmi_client.authn.DBMIUsersModelAuthenticationBackend' in settings.AUTHENTICATION_BACKENDS or \
                    'dbmi_client.authn.DBMIAdminModelAuthenticationBackend' in settings.AUTHENTICATION_BACKENDS
 
     def get_logger(self):
@@ -258,6 +306,15 @@ class DBMISettings(object):
 # Create the instance by which the settings should be accessed
 dbmi_settings = DBMISettings(CONFIG_DEFAULTS)
 
+# Configure logging
+logger = logging.getLogger(dbmi_settings.LOGGER_NAME)
+logger.disabled = not dbmi_settings.ENABLE_LOGGING
+logger.setLevel(dbmi_settings.LOG_LEVEL)
+
+# Set the sentry client if Raven is used
+if 'raven.contrib.django.raven_compat' in settings.INSTALLED_APPS:
+    setattr(settings, 'SENTRY_CLIENT', 'dbmi_client.sentry.DBMISentryClient')
+
 
 def reload_dbmi_settings(*args, **kwargs):
     setting = kwargs['setting']
